#!/usr/bin/pike

int SNL_ERR_OK      =  0;   // no error
int SNL_ERR_OPEN    = -1;   // could'nt open socket
int SNL_ERR_LISTEN  = -2;   // error while listening to socket
int SNL_ERR_CONNECT = -3;   // connecting to socket failed
int SNL_ERR_BIND    = -4;   // couldn't bind to socket
int SNL_ERR_ACCEPT  = -5;   // error while accepting connection
int SNL_ERR_READ    = -6;   // error while reading from socket
int SNL_ERR_WRITE   = -7;   // failed to write to socket
int SNL_ERR_CLOSED  = -8;   // peer closed connection
int SNL_ERR_BUFFER  = -9;   // out of memory (buffer allocation error)
int SNL_ERR_ADDRESS = -10;  // could not get address from hostname

void
thread_read(object fd, function cb, mixed data)
{
	string len, buf;
	int buf_len = 0;

	while (1) {
		len = fd->read(4);
		if (len != "") {
			write(len);
			for (int i=0; i<4; i++) buf_len |= len[i]<<(3-i)*8;
			buf = fd->read(buf_len);
			cb(fd, SNL_ERR_OK, buf, buf_len, data);
		} else {
			cb(fd, SNL_ERR_CLOSED, "", 0, data);
			destruct (fd);
			break;
		}
	}
}

void
thread_listen(int port, function cb, mixed data)
{
	int err = SNL_ERR_OK;
	object socket, fd;

	socket = Stdio.Port(port);
	if (!socket) {
		cb(port, SNL_ERR_LISTEN, fd, data);
		destruct (socket);
		return;
	}
	while (1) {
		fd = socket->accept();
		if (!fd) err = SNL_ERR_ACCEPT;
		cb(port, err, fd, data);
	}
}

int
snl_read(object fd, function cb, void|mixed data)
{
	thread_create(thread_read, fd, cb, data);
	return (SNL_ERR_OK);
}

int
snl_write(object fd, string buf, void|int len)
{
	string data = "";
	int data_len;

	if (zero_type(len)) data_len = strlen(buf); else data_len = len;
	for (int i=0; i<4; i++) data += sprintf("%c", data_len>>((3-i)*8)&0xff);
	if (fd->write(data + buf[..data_len]) != data_len) return (SNL_ERR_WRITE);
	return (SNL_ERR_OK);
}

int
snl_listen(int port, function cb, void|mixed data)
{
	thread_create(thread_listen, port, cb, data);
	return (SNL_ERR_OK);
}

object
snl_connect(string name, int port)
{
	object fd;

	fd = Stdio.File();
	fd->open_socket();
	fd->connect(name, port);
	return (fd);
}
